@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2008  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -32,9 +32,12 @@ extern "C" {
  * @short_description: Functions for logging error and debug information
  */
 
-extern void connman_info(const char *format, ...);
-extern void connman_error(const char *format, ...);
-extern void connman_debug(const char *format, ...);
+extern void connman_info(const char *format, ...)
+				__attribute__((format(printf, 1, 2)));
+extern void connman_error(const char *format, ...)
+				__attribute__((format(printf, 1, 2)));
+extern void connman_debug(const char *format, ...)
+				__attribute__((format(printf, 1, 2)));
 
 /**
  * DBG:
