@@ -279,6 +279,12 @@ static int add_interface(struct supplicant_task *task)
 		return -EIO;
 	}
 
+	if (call == NULL) {
+		connman_error("D-Bus connection not available");
+		dbus_message_unref(message);
+		return -EIO;
+	}
+
 	dbus_pending_call_set_notify(call, add_interface_reply, task, NULL);
 
 	dbus_message_unref(message);
@@ -349,6 +355,12 @@ static int create_interface(struct supplicant_task *task)
 		return -EIO;
 	}
 
+	if (call == NULL) {
+		connman_error("D-Bus connection not available");
+		dbus_message_unref(message);
+		return -EIO;
+	}
+
 	dbus_pending_call_set_notify(call, get_interface_reply, task, NULL);
 
 	dbus_message_unref(message);
@@ -369,6 +381,8 @@ static void remove_interface_reply(DBusPendingCall *call, void *user_data)
 
 	connman_device_unref(task->device);
 
+	inet_ifdown(task->ifindex);
+
 	free_task(task);
 
 	dbus_message_unref(reply);
@@ -401,6 +415,12 @@ static int remove_interface(struct supplicant_task *task)
 		return -EIO;
 	}
 
+	if (call == NULL) {
+		connman_error("D-Bus connection not available");
+		dbus_message_unref(message);
+		return -EIO;
+	}
+
 	dbus_pending_call_set_notify(call, remove_interface_reply, task, NULL);
 
 	dbus_message_unref(message);
@@ -783,6 +803,27 @@ static int initiate_scan(struct supplicant_task *task)
 	return 0;
 }
 
+static char *build_group(const unsigned char *ssid, unsigned int ssid_len,
+					const char *mode, const char *security)
+{
+	GString *str;
+	unsigned int i;
+
+	if (ssid_len < 1)
+		return NULL;
+
+	str = g_string_sized_new((ssid_len * 2) + 24);
+	if (str == NULL)
+		return NULL;
+
+	for (i = 0; i < ssid_len; i++)
+		g_string_append_printf(str, "%02x", ssid[i]);
+
+	g_string_append_printf(str, "_%s_%s", mode, security);
+
+	return g_string_free(str, FALSE);
+}
+
 static void extract_addr(DBusMessageIter *value,
 					struct supplicant_result *result)
 {
@@ -919,8 +960,9 @@ static void properties_reply(DBusPendingCall *call, void *user_data)
 	struct connman_network *network;
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
-	const char *mode, *security;
 	unsigned char strength;
+	const char *mode, *security;
+	char *group;
 
 	DBG("task %p", task);
 
@@ -1011,6 +1053,10 @@ static void properties_reply(DBusPendingCall *call, void *user_data)
 	else
 		security = "none";
 
+	mode = (result.adhoc == TRUE) ? "adhoc" : "managed";
+
+	group = build_group(result.ssid, result.ssid_len, mode, security);
+
 	network = connman_device_get_network(task->device, result.path);
 	if (network == NULL) {
 		int index;
@@ -1028,24 +1074,37 @@ static void properties_reply(DBusPendingCall *call, void *user_data)
 
 		connman_network_set_string(network, "Address", result.addr);
 
+		connman_network_set_group(network, group);
+
+		if (result.name != NULL && result.name[0] != '\0')
+			connman_network_set_string(network, "Name", result.name);
+
+		connman_network_set_uint8(network, "Strength", strength);
+
+		connman_network_set_string(network, "WiFi.Mode", mode);
+		connman_network_set_string(network, "WiFi.Security", security);
+
 		if (connman_device_add_network(task->device, network) < 0) {
 			connman_network_unref(network);
 			goto done;
 		}
 	}
 
+	connman_network_set_group(network, group);
+
+	g_free(group);
+
 	if (result.name != NULL && result.name[0] != '\0')
 		connman_network_set_string(network, "Name", result.name);
 
 	connman_network_set_blob(network, "WiFi.SSID",
 						result.ssid, result.ssid_len);
 
-	mode = (result.adhoc == TRUE) ? "adhoc" : "managed";
 	connman_network_set_string(network, "WiFi.Mode", mode);
 
 	DBG("%s (%s %s) strength %d (%s)",
-			result.name, mode, security, strength,
-			(result.has_wps == TRUE) ? "WPS" : "no WPS");
+				result.name, mode, security, strength,
+				(result.has_wps == TRUE) ? "WPS" : "no WPS");
 
 	connman_network_set_available(network, TRUE);
 	connman_network_set_uint8(network, "Strength", strength);
@@ -1090,6 +1149,12 @@ static void get_properties(struct supplicant_task *task)
 		goto noscan;
 	}
 
+	if (call == NULL) {
+		connman_error("D-Bus connection not available");
+		dbus_message_unref(message);
+		goto noscan;
+	}
+
 	dbus_pending_call_set_notify(call, properties_reply, task, NULL);
 
 	dbus_message_unref(message);
@@ -1181,6 +1246,11 @@ static void scan_results_available(struct supplicant_task *task)
 	if (task->noscan == FALSE)
 		connman_device_set_scanning(task->device, TRUE);
 
+	if (call == NULL) {
+		connman_error("D-Bus connection not available");
+		goto done;
+	}
+
 	dbus_pending_call_set_notify(call, scan_results_reply, task, NULL);
 
 done:
