@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2008  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -43,6 +43,11 @@ static void sig_term(int sig)
 	g_main_loop_quit(main_loop);
 }
 
+static void sig_debug(int sig)
+{
+	__connman_toggle_debug();
+}
+
 static void disconnect_callback(DBusConnection *conn, void *user_data)
 {
 	DBG("D-Bus disconnect");
@@ -51,13 +56,24 @@ static void disconnect_callback(DBusConnection *conn, void *user_data)
 }
 
 static gchar *option_device = NULL;
+static gchar *option_plugin = NULL;
+static gchar *option_nodevice = NULL;
+static gchar *option_noplugin = NULL;
 static gboolean option_detach = TRUE;
 static gboolean option_compat = FALSE;
 static gboolean option_debug = FALSE;
+static gboolean option_selftest = FALSE;
+static gboolean option_version = FALSE;
 
 static GOptionEntry options[] = {
 	{ "device", 'i', 0, G_OPTION_ARG_STRING, &option_device,
-				"Specify network device/interface", "DEV" },
+			"Specify networking device or interface", "DEV" },
+	{ "nodevice", 'I', 0, G_OPTION_ARG_STRING, &option_nodevice,
+			"Specify networking interface to ignore", "DEV" },
+	{ "plugin", 'p', 0, G_OPTION_ARG_STRING, &option_plugin,
+				"Specify plugins to load", "NAME" },
+	{ "noplugin", 'P', 0, G_OPTION_ARG_STRING, &option_noplugin,
+				"Specify plugins not to load", "NAME" },
 	{ "nodaemon", 'n', G_OPTION_FLAG_REVERSE,
 				G_OPTION_ARG_NONE, &option_detach,
 				"Don't fork daemon to background" },
@@ -65,6 +81,10 @@ static GOptionEntry options[] = {
 				"Enable Network Manager compatibility" },
 	{ "debug", 'd', 0, G_OPTION_ARG_NONE, &option_debug,
 				"Enable debug information output" },
+	{ "selftest", 't', 0, G_OPTION_ARG_NONE, &option_selftest,
+				"Run self testing routines" },
+	{ "version", 'v', 0, G_OPTION_ARG_NONE, &option_version,
+				"Show version information and exit" },
 	{ NULL },
 };
 
@@ -95,6 +115,11 @@ int main(int argc, char *argv[])
 
 	g_option_context_free(context);
 
+	if (option_version == TRUE) {
+		printf("%s\n", VERSION);
+		exit(0);
+	}
+
 	if (option_detach == TRUE) {
 		if (daemon(0, 0)) {
 			perror("Can't start daemon");
@@ -102,11 +127,17 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	mkdir(STATEDIR, S_IRUSR | S_IWUSR | S_IXUSR |
-			S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+	if (mkdir(STATEDIR, S_IRUSR | S_IWUSR | S_IXUSR |
+				S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) < 0) {
+		if (errno != EEXIST)
+			perror("Failed to create state directory");
+	}
 
-	mkdir(STORAGEDIR, S_IRUSR | S_IWUSR | S_IXUSR |
-			S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+	if (mkdir(STORAGEDIR, S_IRUSR | S_IWUSR | S_IXUSR |
+				S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) < 0) {
+		if (errno != EEXIST)
+			perror("Failed to create storage directory");
+	}
 
 	main_loop = g_main_loop_new(NULL, FALSE);
 
@@ -140,43 +171,63 @@ int main(int argc, char *argv[])
 
 	__connman_log_init(option_detach, option_debug);
 
-	__connman_storage_init();
+	if (option_selftest == TRUE) {
+		if (__connman_selftest() < 0) {
+			connman_error("Self testing routines failed");
+			goto selftest;
+		}
+	}
 
-	__connman_element_init(conn, option_device);
+	__connman_dbus_init(conn);
+
+	__connman_storage_init();
+	__connman_element_init(conn, option_device, option_nodevice);
 
 	__connman_agent_init(conn);
-
 	__connman_manager_init(conn, option_compat);
-
 	__connman_profile_init(conn);
 
+	__connman_resolver_init();
 	__connman_rtnl_init();
+	__connman_udev_init();
 
-	__connman_plugin_init();
+	__connman_plugin_init(option_plugin, option_noplugin);
+
+	__connman_element_start();
 
 	g_free(option_device);
+	g_free(option_plugin);
+	g_free(option_nodevice);
+	g_free(option_noplugin);
 
 	memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = sig_term;
 	sigaction(SIGINT, &sa, NULL);
 	sigaction(SIGTERM, &sa, NULL);
 
+	sa.sa_handler = sig_debug;
+	sigaction(SIGUSR2, &sa, NULL);
+
 	g_main_loop_run(main_loop);
 
+	__connman_element_stop();
+
 	__connman_plugin_cleanup();
 
+	__connman_udev_cleanup();
 	__connman_rtnl_cleanup();
+	__connman_resolver_cleanup();
 
 	__connman_profile_cleanup();
-
 	__connman_manager_cleanup();
-
 	__connman_agent_cleanup();
 
 	__connman_element_cleanup();
-
 	__connman_storage_cleanup();
 
+	__connman_dbus_cleanup();
+
+selftest:
 	__connman_log_cleanup();
 
 	dbus_connection_unref(conn);
