@@ -27,15 +27,28 @@
 
 #include <glib.h>
 
+#ifdef CONNMAN_PLUGIN_BUILTIN
+#undef CONNMAN_PLUGIN_BUILTIN
+#endif
+
 #include "connman.h"
 
 static GSList *plugins = NULL;
 
 struct connman_plugin {
 	void *handle;
+	gboolean active;
 	struct connman_plugin_desc *desc;
 };
 
+static gint compare_priority(gconstpointer a, gconstpointer b)
+{
+	const struct connman_plugin *plugin1 = a;
+	const struct connman_plugin *plugin2 = b;
+
+	return plugin2->desc->priority - plugin1->desc->priority;
+}
+
 static gboolean add_plugin(void *handle, struct connman_plugin_desc *desc)
 {
 	struct connman_plugin *plugin;
@@ -44,7 +57,7 @@ static gboolean add_plugin(void *handle, struct connman_plugin_desc *desc)
 		return FALSE;
 
 	if (g_str_equal(desc->version, CONNMAN_VERSION) == FALSE) {
-		DBG("version mismatch for %s", desc->description);
+		connman_error("Version mismatch for %s", desc->description);
 		return FALSE;
 	}
 
@@ -53,26 +66,52 @@ static gboolean add_plugin(void *handle, struct connman_plugin_desc *desc)
 		return FALSE;
 
 	plugin->handle = handle;
+	plugin->active = FALSE;
 	plugin->desc = desc;
 
-	if (desc->init() < 0) {
-		g_free(plugin);
-		return FALSE;
-	}
-
-	plugins = g_slist_append(plugins, plugin);
+	plugins = g_slist_insert_sorted(plugins, plugin, compare_priority);
 
 	return TRUE;
 }
 
+static gboolean check_plugin(struct connman_plugin_desc *desc,
+				const char *pattern, const char *exclude)
+{
+	if (exclude != NULL &&
+			g_pattern_match_simple(exclude, desc->name) == TRUE) {
+		connman_info("Excluding %s", desc->description);
+		return FALSE;
+	}
+
+	if (pattern != NULL &&
+			g_pattern_match_simple(pattern, desc->name) == FALSE) {
+		connman_info("Ignoring %s", desc->description);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+#include "builtin.h"
+
 int __connman_plugin_init(const char *pattern, const char *exclude)
 {
+	GSList *list;
 	GDir *dir;
 	const gchar *file;
 	gchar *filename;
+	unsigned int i;
 
 	DBG("");
 
+	for (i = 0; __connman_builtin[i]; i++) {
+		if (check_plugin(__connman_builtin[i],
+						pattern, exclude) == FALSE)
+			continue;
+
+		add_plugin(NULL, __connman_builtin[i]);
+	}
+
 	dir = g_dir_open(PLUGINDIR, 0, NULL);
 	if (dir != NULL) {
 		while ((file = g_dir_read_name(dir)) != NULL) {
@@ -87,8 +126,8 @@ int __connman_plugin_init(const char *pattern, const char *exclude)
 
 			handle = dlopen(filename, RTLD_NOW);
 			if (handle == NULL) {
-				g_warning("Can't load %s: %s", filename,
-								dlerror());
+				connman_error("Can't load %s: %s",
+							filename, dlerror());
 				g_free(filename);
 				continue;
 			}
@@ -97,21 +136,13 @@ int __connman_plugin_init(const char *pattern, const char *exclude)
 
 			desc = dlsym(handle, "connman_plugin_desc");
 			if (desc == NULL) {
-				g_warning("Can't load symbol: %s", dlerror());
+				connman_error("Can't load symbol: %s",
+								dlerror());
 				dlclose(handle);
 				continue;
 			}
 
-			if (exclude != NULL && g_pattern_match_simple(exclude,
-							desc->name) == TRUE) {
-				DBG("excluding %s", desc->description);
-				dlclose(handle);
-				continue;
-			}
-
-			if (pattern != NULL && g_pattern_match_simple(pattern,
-							desc->name) == FALSE) {
-				DBG("ignoring %s", desc->description);
+			if (check_plugin(desc, pattern, exclude) == FALSE) {
 				dlclose(handle);
 				continue;
 			}
@@ -123,6 +154,15 @@ int __connman_plugin_init(const char *pattern, const char *exclude)
 		g_dir_close(dir);
 	}
 
+	for (list = plugins; list; list = list->next) {
+		struct connman_plugin *plugin = list->data;
+
+		if (plugin->desc->init() < 0)
+			continue;
+
+		plugin->active = TRUE;
+	}
+
 	return 0;
 }
 
@@ -135,10 +175,11 @@ void __connman_plugin_cleanup(void)
 	for (list = plugins; list; list = list->next) {
 		struct connman_plugin *plugin = list->data;
 
-		if (plugin->desc->exit)
+		if (plugin->active == TRUE && plugin->desc->exit)
 			plugin->desc->exit();
 
-		dlclose(plugin->handle);
+		if (plugin->handle != NULL)
+			dlclose(plugin->handle);
 
 		g_free(plugin);
 	}
